/** UNAS API
 * UNASDesktop API UNAS.js
 * Version 1.0.0 for U-NAS OS V3.0
 * Copyright (C) WanYou - All Rights Reserved
 */

/**
 *Define UNAS namespace
 *@namespace
 */
var UNAS = UNAS || {};
UNAS.Version = '1.0.0'; //API Version
UNAS.LoginUser = ''; //the logged in user name

UNAS.Apps = []; //Loaded Apps, an array of AppInfo
UNAS.AppGroups = {}; //a collection of App Groups. UNAS.AppGroups.XYZ is XYZ AppGroup. Created automatically through API RegisterToAppGroup();
UNAS.AppGroupsAppNames = {};
UNAS.AppLoaded = false; //AppLoaded State
UNAS.ElementCanDrag = true; //Desktop Element can drag or not
UNAS.ElementSortType = "gridAlign";
UNAS.CMGridAlignls = {}; //grid align sort ultimately object
UNAS.DesktopNavbarImgHeight = 32; //nvabar img width and height
UNAS.DesktopNavbarImgWidth = 32;
UNAS.DesktopNavbarIconWidth = 40; //navbar icon width and height
UNAS.DesktopNavbarIconHeight = 38;
UNAS.DesktopDropTarget = "pageWrapper";
UNAS.NavbarSiteNum = {}; // navbar elements position info
UNAS.IsDropTarget = "false";
UNAS.isAppCountAllLoad = 1;
UNAS.isInit = false;
UNAS.isDesktopReady = false;
UNAS.UngridAlignApps = [];
UNAS.objCount = 0;
UNAS.AppCanClick = true; //双击时，避免因网络延迟出现两个DockTab;
UNAS.Clipboard = [];  //剪贴板，ufile用
UNAS.ConfigWizardOobe = {};

UNAS.Desktop = {}; //UNAS Desktop, everything related to Desktop
UNAS.Desktop.VirtualDesktops = []; //U-NAS supports Multiple Virutual Desktop, Use CreateVirtualDesktop() API to add VD
UNAS.Desktop.CurrentVirtualDesktop = 0; //Index of current virtual desktop
UNAS.Desktop.CurrentLanguage = "English"; //Current language of the Desktop
UNAS.Desktop.Theme = {
  Name: 'default',
  Dir: 'default',
  cssFiles: []
}; //Desktop Theme, cssFiles:[{element:, file:}]
UNAS.Desktop.Wallpaper = 'desktop/themes/default/wallpaper.png'; //Wallpaper file, in referrenc to '/desktop/wallpapers/'
UNAS.Desktop.Wallpaper.fit = 'fill'; //Wallpaper fit. default is fill.

// UNAS Translation related
UNAS.PreviousLanguage = "English"; //Previous language after language changes
UNAS.CurrentLanguage = "English"; //Current language
UNAS.SupportedLanguages = ["English"]; //List of supported languages
UNAS.TranslationTable = {}; //The BIG Translation table, use property rather than array
UNAS.LoadedTranslationFiles = []; //An array of the translation files loaded

/**
 *Get all nodes under an element/node. Non-recursive. need optimizaiton.
 *@param {Element}  node - root element
 *@return {Elements} An array of elements
 */
UNAS.NodesUnder = function(node) {
  var elems = [];
  var array = [],
    i = 0,
    k = 0,
    elementCount = 0,
    len = 0,
    childNodes, item;
  while (node != null) {
    //console.log(node.tagName);
    childNodes = node.childNodes;
    len = node.childNodes.length;
    elementCount = 0;
    if (len > 0) {
      for (i = 0; i < len; i++) {
        item = childNodes[i];
        elems.push(item);
        if (item.nodeType === 1) {
          elementCount++;
          node = item;
          break;
        }
      }
      for (k = len - 1; k > i; k--) {
        item = childNodes[k];
        elems.push(item);
        if (item.nodeType == 1) {
          elementCount++;
          array.push(item);
        }
      }
      if (elementCount < 1) {
        node = array.pop();
      }
    } else {
      node = array.pop();
    }
  }
  return elems;
}
/**
 *Save User's desktop information on server, under user's data (LDAP)
 */
UNAS.SaveUserDesktop = function() {
  //collect information needs to be saved, convert to string, and do an ajax save.
  var desktopInfo = {
    'language': UNAS.CurrentLanguage,
    'theme': UNAS.Desktop.Theme.Dir,
    'wallpaper': UNAS.Desktop.Wallpaper,
    'fit': UNAS.Desktop.Wallpaper.fit,
    'sortType': UNAS.ElementSortType,
    'screenSize': UNAS.GetDesktopPositionInfo("pageWrapper"),
    'shortcuts': []
  };
  var parentElement = null;
  var exp = new Date();
  exp.setTime(exp.getTime() + (86400 * 1000 * 30));
  document.cookie = "login_lang=" + window.btoa(escape(UNAS.CurrentLanguage)) + "; expires=" + exp
    .toGMTString() + "; path=/";

  // console.log(deWeightFour(UNAS.Desktop.VirtualDesktops));
  for (var i = 0; i < UNAS.Desktop.VirtualDesktops.length; i++) {
    for (var j = 0; j < UNAS.Desktop.VirtualDesktops[i].Shortcuts.length; j++) {
      var st = UNAS.Desktop.VirtualDesktops[i].Shortcuts[j]
      if (st.parentElement) {
        parentElement = st.parentElement;
      }
      var img = st.getElementsByTagName("IMG");
      var span = st.getElementsByTagName("SPAN");
      desktopInfo.shortcuts.push({
        vd: i,
        id: st.id,
        left: st.style.left,
        top: st.style.top,
        width: st.style.width,
        height: st.style.height,
        imgWidth: img[0].style.width,
        imgHeight: img[0].style.height,
        spanDisplay: span[0].style.display,
        // parentID: st.parentElement.id
        parentID:parentElement.id
      });
    }
  }
  if (UNAS.ElementSortType == "ungridAlign") {
    UNAS.UngridAlignApps = desktopInfo.shortcuts;
  }
  var desktopStr = JSON.stringify(desktopInfo); //desktop info to be saved.
  var req = new Request({
    url: '/api/php/UNASJS.php',
    method: 'POST',
    data: {
      'DesktopState': desktopStr,
      'action': 'SaveDesktop'
    },
    onRequest: function() {},
    onSuccess: function(responseText) { //return null or 1
      if (!responseText)
        alert(UNAS._('Save Desktop failed!'));
    },
    onFailure: function() {
      //alert(UNAS._('Save Desktop failed!'));     //导致uncaught exception: out of memory, 堆栈错误
    }
  });
  req.send();
}
/**
 *Load User's desktop
 *call the callback function after user desktop is loaded
 *@param {Function} callback - Callback function, invoked after User Desktop is loaded
 */
UNAS.LoadUserDesktop = function(callback) {
  //note: Admin may change things, such as remove wallpaper/theme, so LoadUserDesktop may cause problem!
  //Ajax get saved desktop and convert to
  var req = new Request({
    url: '/api/php/UNASJS.php',
    method: 'POST',
    data: {
      'action': 'LoadDesktop'
    },
    onRequest: function() {},
    onSuccess: function(responseText) {
      if (responseText == null || responseText.length == 0) {
        //cannot get the saved desktop
      } else {
        var desktopInfo = JSON.parse(responseText);
        if (desktopInfo != null) {
          var jsSrc = (navigator.language || navigator.browserLanguage).toLowerCase();
          var lan = UNAS.CurrentLanguage;
          if (jsSrc != 'zh-cn') {
            lan = 'English';
          }else {
            lan = '简体中文';
          }
          if (getCookie('login_lang')) {
            lan = unescape(window.atob(getCookie('login_lang')));
          }else {
            lan = sessionStorage.getItem('tempLang') || lan;
          }

          if (desktopInfo.language && desktopInfo.language != "") {
            lan = desktopInfo.language;
          }

          UNAS.CurrentLanguage = lan; //need to fire ChangeLanguage event to update the language
          setTimeout(function(){
          var exp = new Date();
					exp.setTime(exp.getTime() + (86400 * 1000 * 30));
					document.cookie = "login_lang=" + window.btoa(escape(lan)) + "; expires=" + exp
						.toGMTString() + "; path=/";
          }, 1000)
          UNAS.LoadTheme(desktopInfo.theme);
          UNAS.SetWallpaper(desktopInfo.wallpaper);
          // for (var i = 0; i < UNAS.Desktop.VirtualDesktops.length; i++) {
          // for (var j = 0; j < UNAS.Desktop.VirtualDesktops[i].Shortcuts.length; j++) {
          // var st = UNAS.Desktop.VirtualDesktops[i].Shortcuts[j];
          desktopInfo.shortcuts = eval(desktopInfo.shortcuts);
          jQuery('#pageWrapper > div.abs').html("");
          UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].Shortcuts = [];
          for (var k = 0; k < desktopInfo.shortcuts.length; k++) {
            // if (desktopInfo.shortcuts[k].id == st.id) { //found the desktop shortcut based on id
            var appName = desktopInfo.shortcuts[k].id.replace("DesktopShortcut", "");
            var appInfo = UNAS.GetAppInfo(appName);
            if (appInfo) {
              var st = document.getElementById(desktopInfo.shortcuts[k].id);
              if (!st) {
                UNAS.DesktopShortcut(appName, appInfo.appIcon, UNAS._(appName), appInfo.appStartFunc);
                st = document.getElementById(desktopInfo.shortcuts[k].id);
              }
              var img = st.getElementsByTagName("IMG");
              var span = st.getElementsByTagName("SPAN");
              st.style.left = desktopInfo.shortcuts[k].left;
              st.style.top = desktopInfo.shortcuts[k].top;
              st.style.width = desktopInfo.shortcuts[k].width;
              st.style.height = desktopInfo.shortcuts[k].height;
              img[0].style.width = desktopInfo.shortcuts[k].imgWidth;
              img[0].style.height = desktopInfo.shortcuts[k].imgHeight;
              span[0].style.display = desktopInfo.shortcuts[k].spanDisplay;
              if (st.parentElement.id != desktopInfo.shortcuts[k].parentID) { //shortcut is not on desktop
                var elem = document.getElementById(desktopInfo.shortcuts[k].parentID);
                if (elem != null) {
                  st.parentNode.removeChild(st);
                  elem.appendChild(st); //move to the parent
                }
              }
              // }
            } else {
              continue;
            }
          }
          // }
          // }
          var st = desktopInfo.sortType || UNAS.ElementSortType;
          contextMenuFun._selectedIcon(st, jQuery(document.getElementsByName(st)));
          if (st == "ungridAlign") { //非网格排序
            UNAS.ElementCanDrag = true; //启用拖动
            UNAS.ElementSortType = st;
            UNAS.UngridAlignApps = desktopInfo.shortcuts;
          } else if (st == "autoArrange") {
            UNAS.ElementSortType = st;
            var dstpInfo = desktopInfo.screenSize || UNAS.GetDesktopPositionInfo("pageWrapper");
            contextMenuFun.__gridGenerateOrderFun(dstpInfo);
            contextMenuFun.__generateListOrder();
            UNAS.ElementCanDrag = false; //禁用拖动
            UNAS.__autoArrangeFun(dstpInfo);
            // if(jQuery(document.getElementById("pageWrapper")).width() != dstpInfo.Sw){
            // }
          } else if (st == "gridAlign") {
            UNAS.ElementCanDrag = true; //启用拖动
            UNAS.ElementSortType = st;
            var dstpInfo = desktopInfo.screenSize || UNAS.GetDesktopPositionInfo("pageWrapper");
            contextMenuFun.__gridGenerateOrderFun(dstpInfo);
            if (!desktopInfo.sortType && !desktopInfo.screenSize) {
              contextMenuFun.__generateListOrder();
            }
            UNAS.__gridAlignFun(dstpInfo);
            // if(jQuery(document.getElementById("pageWrapper")).width() != dstpInfo.Sw){
            // }
          }
        }
        if (callback && typeof callback === "function") { //callback when load is done
          callback();
        }
      }
    },
    onFailure: function() {
      alert(UNAS._('Load Desktop failed!'));
    }
  });
  req.send();

}

/**
 *Get a list of themes available
 *call the callback function to process the returned theme list
 *@param {Function} callback - Callback function, invoked after theme list is loaded
 */
UNAS.GetThemeList = function(callback) {
  var req = new Request({
    url: '/api/php/UNASJS.php',
    method: 'POST',
    data: {
      'action': 'GetThemeList'
    },
    onRequest: function() {},
    onSuccess: function(responseText) {
      if (!responseText || responseText.length <= 0) {
        //not found
      } else {
        var themeList = JSON.parse(responseText);
        if (callback && typeof callback === "function") {
          callback();
        }
      }
    },
    onFailure: function() {
      alert(UNAS._('Loading Theme List failed!'));
    }
  });
  req.send();
}
/**
 *Load Theme by name (directory)
 *@param {String} dir - Theme name, (the directory name on the server)
 *@param {Function} callback - Callback function, invoked after theme is loaded
 *
 */
UNAS.LoadTheme = function(dir, callback) {
  if (typeof dir !== 'undefined') {
    //load theme css files via Ajax
    var req = new Request({
      url: '/api/php/UNASJS.php',
      method: 'POST',
      data: {
        'themeDir': dir,
        'action': 'GetTheme'
      },
      onRequest: function() {},
      onSuccess: function(responseText) {
        if (responseText == null || responseText.length == 0) {

        } else {
          var themeCSSList = JSON.parse(responseText);
          if (themeCSSList != null) {
            //first remove all old css files.
            for (var i = 0; i < UNAS.Desktop.Theme.cssFiles.length; i++) {
              if (UNAS.Desktop.Theme.cssFiles[i].element != null) {
                UNAS.Desktop.Theme.cssFiles[i].element.parentNode.removeChild(UNAS.Desktop.Theme.cssFiles[i].element); //remove one theme css file.
              }
            }
            UNAS.Desktop.Theme.cssFiles = []; //clear all
            //add new css files
            var themeCSSCount = themeCSSList.length;
            for (var i = 0; i < themeCSSList.length; i++) {
              var elem = UNAS.LoadCSS(themeCSSList[i], function() {
                themeCSSCount--;
                if (themeCSSCount <= 0) { //now all apps loaded
                  var timer = function() {
                    if (UNAS.isDesktopReady) {
                      jQuery('#unas_desktop_waiting_content').attr("style", "display:none");
                      jQuery('#unas_desktop_waiting_logo').attr("style", "display:none");
                      clearTimeout(timer);
                      UNAS.ConfigWizardOobe();
                    } else {
                      setTimeout(timer, 50);
                    }
                  };
                  timer();
                }
              });
              UNAS.Desktop.Theme.cssFiles.push({
                element: elem,
                file: themeCSSList[i]
              }); //keep a record

            }
            UNAS.Desktop.Theme.Dir = dir;
            UNAS.Desktop.Theme.Name = dir;
            if (callback && typeof callback === "function") { //load theme is done, callback
              callback();
            }
          }
        }
      },
      onFailure: function() {
        alert(UNAS._('Loading Theme failed!'));
      }
    });
    req.send();
  }
}
/**
 *Get the DesktopDock, 'status bar'
 *@returns {Element} The Desktop Dock element
 */
UNAS.GetDesktopDock = function() {
  return document.getElementById('dockWrapper');
}
/**
 * Get desktopHeader, 'top bar'
 *@returns {Element} The Desktop Header element
 */
UNAS.GetDesktopHeader = function() {
  return document.getElementById('desktopHeader');
}
/**
 *Get Desktop Wrapper， （it is the wrapper DIV of all Virtual Desktops, 'desktop canvas'
 *@returns {Element} The Desktop Canvas element
 */
UNAS.GetDesktopWrapper = function() {
  return document.getElementById('pageWrapper');
}
/**
 *Get Current Virtual Desktop DIV, the big wrapper.
 *@returns {Element} The Desktop Virtual Desktop DIV element
 */
UNAS.GetCurrentVirtualDesktopDIV = function() {
  return UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].DIV;
}
/**
 *UNAS Create Virtual Desktop
 *Creates and adds a Virtual Desktop to the array of UNAS.Desktop.VirtalDesktops
 *@param {String} id - Virutal Deskto id
 *@param {String} name - Name of the Virtual Desktop
 */
UNAS.CreateVirtualDesktop = function(id, name) {
  var vd = { //shoudl we define this in Desktop Definition file? to support customize
    DIV: document.createElement("div"),
    VirtualDesktopName: name || '', //default name is empty
    Left0: 20,
    Top0: 20,
    ShortcutLeft: 20,
    ShortcutTop: 20,
    ShortcutWidth: 80,
    ShortcutHeight: 100,
    ShortcutDeltaX: 80,
    ShortcutDeltaY: 110,
    ShortcutIconSize: 48,
    ShortcutIconsPerColumn: 6,
    Shortcuts: []
  };
  vd.id = id || 'unas-virtual-desktop-' + UNAS.Desktop.VirtualDesktops.length; //default id;
  vd.DIV.className = 'abs'; //CHANGE THIS Class NAME in Theme file！！！！！
  vd.DIV.style.visibility = 'hidden'; //initially hidden
  if (typeof UNAS.GetDesktopWrapper() != 'undefined') {
    UNAS.GetDesktopWrapper().appendChild(vd.DIV); //insert into the desktop
    UNAS.Desktop.VirtualDesktops.push(vd); //add it in
  }
}
/**
 *Show a Virutal Desktop
 *@param {Integer} i - Index of the Virtual Destkop
 */
UNAS.ShowVirtualDesktop = function(i) {
  if (typeof i === 'undefined') i = UNAS.Desktop.CurrentVirtualDesktop;
  if (i >= 0 && i < UNAS.Desktop.VirtualDesktops.length) {
    for (var j = 0; j < UNAS.Desktop.VirtualDesktops.length; j++)
      UNAS.Desktop.VirtualDesktops[j].DIV.style.visibility = 'hidden'; //hide all VDs
    UNAS.Desktop.CurrentVirtualDesktop = i;
    UNAS.Desktop.VirtualDesktops[i].DIV.style.visibility = 'visible';
    MUI.Windows.windowOptions.container = UNAS.Desktop.VirtualDesktops[i].DIV; //make this Virtual Desktop the container of windows.  MochaUI is BUGGY on drag window, need fix.
  }
}
/**
 *UNAS Translate a string
 *@param {String} str - English ONLY text, could have marks for id
 *@param {String} lang - The target langauge,use UNAS.CurrentLanguage if not provided
 *@return  {String} Returns the translated text, if a translation does not exist return the original text
 * example: UNAS._('Hello'); //simple text, no duplicate translations
 * example: UNAS._('##$@@$##Hello##$@@$##');
 * example: UNAS._('##$@1@$##Hello##$@1@$##');
 * example: UNAS._('##$@1@$##Hello##$@1@$##' World!); //only part of the string needs to be translated
 * example: UNAS._('Hi, ##$@1@$##Hello##$@1@$## World. ##$@1@$##Welcome##$@1@$## to U-NAS world.);
 */
UNAS._ = function(str, lang) { //Translate
  var lead = '##$@'; //lead mark
  var end = '@$##'; //end mark
  var L = end.length;
  if (typeof(str) === 'undefined') return ""; //
  lang = lang || UNAS.CurrentLanguage;
  //just find all the sub strings need to be translated.
  var ss = str;
  var subStr = '';
  var lPos = ss.indexOf(lead);
  var ePos = 0;
  var count = 0;
  var stringLength = ss.length;

  if (lPos > -1) {
    do {
      var Marker = '';
      var Marker2 = '';
      var id = '';
      subStr = '';
      if (lPos > -1) {
        //found lead mark
        ePos = ss.indexOf(end); //find end mark, MUST Have.
        if (ePos > -1) {
          subStr = ss.substring(lPos, ePos + L);
          Marker = ss.substring(lPos, ePos + L);
          id = subStr.replace(lead, ''); //remove lead mar
          id = id.replace(end, ''); //remove end mark and now id is retrieved
          id = id.trim(); //remove whitespaces from both ends
          var tmpStr = ss.substring(ePos + L, ss.length); //the rest of the string
          var A = tmpStr.indexOf(lead);
          var B = tmpStr.indexOf(end); //find the closing part .
          if (B > -1) {
            Marker2 = tmpStr.substring(A, B + L); //end mark
            subStr = subStr + tmpStr.substring(0, B + L); //found subStr to translate
          } else {
            //no close mark, to end
            subStr = subStr + tmpStr; //found subStr to translate
          }
        } else {
          break; //something is wrong, don't translate.
        }
      }
      var text = subStr.replace(Marker, ''); //remove 'lead+id+end',from both ends; this is the real text need to be translated.
      text = text.replace(Marker2, ''); //remove end..
      if (id.length == 0) {
        id = '0'; //default if no id is specifed.
      }
      var stest = JSON.stringify(text + lang + id); //test only, remove it after testing
      var tmp2 = UNAS.TranslationTable[JSON.stringify(text + lang + id)]; //translate to lang.
      if (tmp2) { //found translation
        ss = ss.replace(subStr, tmp2); //translate the orignial text
      } else {
        ss = ss.replace(subStr, text); //remove the mark, use original text (substring)
      }
      lPos = ss.indexOf(lead);
      count++;
    } while (lPos > -1 && count <= stringLength);
  } else {
    //no mark, translate the whole text
    var tmp3 = UNAS.TranslationTable[JSON.stringify(ss + lang + '0')];
    if (tmp3) ss = tmp3;
  }
  return ss;
}
/**
 *Find Text nodes under a node
 *@param {node} node - the root node to be searched
 *@returns {Nodes} An array of nodes
 */
UNAS.TextNodesUnder = function(node) {
  var all = [];
  for (node = node.firstChild; node; node = node.nextSibling) {

    if (node.nodeType == 3 && jQuery(node).attr('translation') != '0')
      all.push(node);
    else
      if (jQuery(node).attr('translation') != '0') {
        all = all.concat(UNAS.TextNodesUnder(node));
      }
  }
  return all;
}

/**
 *UNAS Load Translation files, fire 'SupportedLanguagesChange' event to notify.
 *@function Dynamically load translation files
 *@param {String} f - File name to be loaded
 *@param {Function} callback - Callback function after loading completes
 */
UNAS.LoadTranslation = function(f, callback) {
  if (!Array.indexOf) {
    Array.prototype.indexOf = function(obj) {
      for (var i = 0; i < this.length; i++) {
        if (this[i] == obj) {
          return i;
        }
      }
      return -1;
    }
  }
  if (typeof f === 'undefined' || UNAS.LoadedTranslationFiles.indexOf(f) >= 0) return; //return if no file or exists
  //Use Ajax to load the file
  var xhr = (window.ActiveXObject) ? new ActiveXObject("Microsoft.XMLHTTP") : (XMLHttpRequest && new XMLHttpRequest()) || null;
  if (xhr) {
    xhr.open('GET', f, true); //must be synchronous....
    xhr.onreadystatechange = function() {
      if (xhr.readyState === 4) {
        if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304) {
          var langs = JSON.parse(xhr.responseText);
          var langCount = UNAS.SupportedLanguages.length;
          if (langs != null && typeof langs.UNASTranslation != null) {
            for (var i = 0; i < langs.UNASTranslation.length; i++) { //array of several languages.
              if (UNAS.SupportedLanguages.indexOf(langs.UNASTranslation[i].language) < 0) UNAS.SupportedLanguages.push(langs.UNASTranslation[i].language); //added to supportedLanguages
              var tmp = langs.UNASTranslation[i].table;
              for (var j = 0; j < tmp.length; j++) {
                var EnglishText = JSON.stringify(tmp[j][0]).slice(1, -1); //original English Text
                var tid = JSON.stringify(tmp[j][1]).slice(1, -1); //text id
                var TranslatedText = JSON.stringify(tmp[j][2]).slice(1, -1);
                UNAS.TranslationTable[JSON.stringify(EnglishText + langs.UNASTranslation[i].language + tid)] = TranslatedText; //use property rather than array to save the translations! [English -->Lang]
                UNAS.TranslationTable[JSON.stringify(TranslatedText + 'English' + tid)] = EnglishText; //use property rather than array to save the translations! [Tanslated -->English]
              }
            }
            UNAS.LoadedTranslationFiles.push(f); //add this file into the loaded list to avoid duplicated loading.
            if (langCount != UNAS.SupportedLanguages.length) {
              UNAS.Event.fireEvent('SupportedLanguagesChange'); //fire 'SupportedLanguagesChange' event to notify
            }
            if (callback && typeof callback === "function") {
              callback();
            }
          }
        } else {
          alert('error:' + xhr.status);
        }
      } else {
        //alert('Loading Translation file failed!');
      }
    }
    xhr.send();
  }
}
/**
 *GetWindow by window id
 *@param {String} id - Window id
 *@returns {Window Instance} or null if not found
 */
UNAS.GetWindow = function(id) {
  if (typeof id === 'undefined') return null;
  var win = null;
  // get all the window instances
  var instances = MUI.Windows.instances;
  instances.each(function(instance) {
    if (instance.options.id === id) {
      win = instance;
      return win;
    }
  });
  return win;
}
/**
 *SetWindowTitle based window id
 *@param {String} id - Window id
 *@param {String} t - Title for the Window
 */
UNAS.SetWindowTitle = function(id, t) {
  var w = UNAS.GetWindow(id);
  if (w != null) {
    w.titleEl.set('html', t);
  }
}

/**
 *Translate the page of a window.
 *call this at the end of the page, not recommend to use
 *@param {String} window_id - Window id
 */
UNAS.TranslatePage = function(window_id) {
  if (typeof window_id === 'undefined') return;
  var appWin = UNAS.GetWindow(window_id);
  var pageTranslators = [];

  //Create an array of Node Translators
  if (appWin) {
    pageTranslators = UNAS.CreateNodesTranslators(appWin.windowEl);
  }
  //handle Change Language event,use the array of translators.
  var OnChangeLanguage = function(e) {
    //we have pageTranslators, just loop it.
    for (var i = 0; i < pageTranslators.length; i++) {
      var node = pageTranslators[i].node; // original node
      var p = pageTranslators[i].translateProperty; //the attribues need to be translated
      if (typeof node[p] === 'undefined') {
        node.setAttribute(p, UNAS._(pageTranslators[i].originalText));
      } else {
        node[p] = UNAS._(pageTranslators[i].originalText); //translate
      }
    }
    //your additional code for Language Change...
  };

  OnChangeLanguage(); //change language on loading page
  //add ChangeLanuage event on loading.
  UNAS.Event.addEvent('ChangeLanguage', OnChangeLanguage);
  // remove OnChangeLanguage event the app window is closed
  addEventSimple(window, 'unload', function(e) {
    UNAS.Event.removeEvent('ChangeLanguage', OnChangeLanguage);
  }, false);
}
/**
 *UNAS.NodeTranslators Creates an array of translators for a root node
 *@constructor, returns an array of NodeTranslator for a node
 *@param {node} rootnode - The root node
 *@returns {Array} an array of translators for a root node
 */
UNAS.CreateNodesTranslators = function(rootnode) {
  //first get all textnodes
  var translators = [];
  var textnodes = UNAS.TextNodesUnder(rootnode);
  for (var i = 0; i < textnodes.length; i++) {
    if (textnodes[i].nodeValue && textnodes[i].nodeValue.length > 0) {
      var nodeTranslator = {};
      nodeTranslator.node = textnodes[i]; //keep a track of node, reference of textnodes
      nodeTranslator.originalText = textnodes[i].nodeValue; //keep a copy of orginal text, maybe not needed.
      nodeTranslator.nodeType = textnodes[i].nodeType; //textnode..
      nodeTranslator.translateProperty = 'nodeValue';
      translators.push(nodeTranslator);
    }
  }
  //then handle all tags (textnode is not a tag)
  var tags = rootnode.getElementsByTagName('*');
  // console.log(tags);
  for (var i = 0; i < tags.length; i++) {
    if (tags[i].title && tags[i].title.length > 0 && tags[i].getAttribute('translation') != 0) {
      var tagTranslator = {};
      tagTranslator.node = tags[i]; //keep a track of node, reference of textnodes
      tagTranslator.originalText = tags[i].title; //keep a copy of orginal text, maybe not needed.
      tagTranslator.nodeType = tags[i].nodeType; //record node type
      tagTranslator.translateProperty = 'title';
      translators.push(tagTranslator);
    }
  }
  //inputs
  var inputs = rootnode.getElementsByTagName("input"); //get all inputs under rootnode
  for (var i = 0; i < inputs.length; i++) {
    var tagTranslator = {};
    tagTranslator.node = inputs[i]; //keep a track of node, reference of textnodes
    if ((inputs[i].type == "button") || (inputs[i].type == "submit"))
      tagTranslator.originalText = inputs[i].value; //keep a copy of orginal text, maybe not needed.
    tagTranslator.nodeType = inputs[i].nodeType; //record node type..
    if ((inputs[i].type == "button") || (inputs[i].type == "submit"))
      tagTranslator.translateProperty = 'value';
    if (inputs[i].placeholder) {
      tagTranslator.originalText = inputs[i].placeholder;
      tagTranslator.translateProperty = 'placeholder';
    }
    translators.push(tagTranslator);
  };
  //options
  //select tags may contain options. could have more than one select tags. actually just find all <option> nodes
  var options = rootnode.getElementsByTagName("option"); //get all options under rootnode
  for (var i = 0; i < options.length; i++) {
    if (options[i].getAttribute('translation') != 0) {
    var tagTranslator = {};
    tagTranslator.node = options[i]; //keep a track of node, reference of textnodes
    tagTranslator.originalText = options[i].text; //keep a copy of orginal text, maybe not needed.
    tagTranslator.nodeType = options[i].nodeType; //record node type..
    tagTranslator.translateProperty = 'text';
    translators.push(tagTranslator);
    }
  };
  //<a> href
  //select tags may contain <a> tags.
  var ATags = rootnode.getElementsByTagName("a"); //get all options under rootnode
  for (var i = 0; i < ATags.length; i++) {
    var tagTranslator = {};
    tagTranslator.node = ATags[i]; //keep a track of node, reference of textnodes
    if (ATags[i].href == null || ATags[i].href.length == 0) {

    } else {
      tagTranslator.originalText = ATags[i].href; //keep a copy of orginal href
      tagTranslator.translateProperty = 'href';
    }
    tagTranslator.nodeType = ATags[i].nodeType; //record node type..
    translators.push(tagTranslator);
  };
  //<img> src
  //select imgs may contain <a> tags.
  var imgs = rootnode.getElementsByTagName("img"); //get all options under rootnode
  for (var i = 0; i < imgs.length; i++) {
    var tagTranslator = {};
    tagTranslator.node = imgs[i]; //keep a track of node, reference of textnodes
    tagTranslator.originalText = imgs[i].src; //keep a copy of orginal href
    tagTranslator.nodeType = imgs[i].nodeType; //record node type..
    tagTranslator.translateProperty = 'src';
    translators.push(tagTranslator);
  };
  return translators;
}

/**
 * Check if an app is registered (through app name,not package name)
 *@param {String} name - App Name to be checked
 */
UNAS.IsAppRegistered = function(name) {
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == name) {
      return true;
    }
  }
  return false;
}

/**
 * 渲染桌面左上角菜单栏
 */
UNAS.renderMenuPanel = function() {
  if (UNAS.Apps != {} && UNAS.AppGroups != {} && UNAS.AppGroupsAppNames != {}) {
    UNAS.menuAppsObj = [], applistStr = '';
    for (var i = 0; i < UNAS.Apps.length; i++) {
      if (!(UNAS.Apps[i].appName in UNAS.AppGroupsAppNames)) {
        UNAS.menuAppsObj.push(UNAS.Apps[i]);
        applistStr += '<div class="li-app" title="' + UNAS._(UNAS.Apps[i].appName) + '" id="' + UNAS.Apps[i].appName + 'Menu">' +
          '<img src="' + UNAS.Apps[i].appIcon + '" alt="error">' +
          '<p>' + UNAS._(UNAS.Apps[i].appName) + '</p>' +
          '</div>';
      }
    }
    jQuery("#appMenulist").append(applistStr);
    jQuery(".appListBox .list").append(applistStr);
    jQuery(".li-app").on('click', function(e) {
      e.stopPropagation();
      jQuery('#desktopHiddenModal').trigger('mousedown');
      UNAS.GetAppInfo(e.currentTarget.id.replace('Menu', '')).appStartFunc();
    })
  }
}
/**
 * 安装时增加左上角菜单栏dom
 */
UNAS.AddMenuPanel = function(appName) {
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == appName && !(UNAS.Apps[i].appName in UNAS.AppGroupsAppNames)) {
      jQuery("#appMenulist").append('<div class="li-app" title="' + UNAS._(UNAS.Apps[i].appName) + '" id="' + UNAS.Apps[i].appName + 'Menu">' +
        '<img src="' + UNAS.Apps[i].appIcon + '" alt="error">' +
        '<p>' + UNAS._(UNAS.Apps[i].appName) + '</p>' +
        '</div>');
      jQuery(document.getElementById(appName + "Menu")).on('click', function(e) {
        e.stopPropagation();
        jQuery('#desktopHiddenModal').trigger('mousedown');
        UNAS.GetAppInfo(e.currentTarget.id.replace('Menu', '')).appStartFunc();
      })
    }
  }
}

/**
 * Load Apps
 *@param {Function} callback - Callback function to be invoked after All Apps are loaded
 */
UNAS.LoadApps = function(callback) {
  var req = new Request({ // Use Mootools ajax to get apps
    url: '/api/php/UNASJS.php',
    method: 'POST',
    data: {
      'action': 'GetApps'
    },
    onRequest: function() {},
    onSuccess: function(responseText) {
      if (responseText == null || responseText.length == 0) {} else {
        UNAS.Apps = JSON.parse(responseText);
        if (UNAS.Apps != null) {
          UNAS.isAppCountAllLoad = UNAS.Apps.length; //unloaded App count
          if (UNAS.isAppCountAllLoad <= 0) { //no apps, callback
            if (callback && typeof callback === "function") {
              callback();
            }
          }
          for (var i = 0; i < UNAS.Apps.length; i++) {
            var app = UNAS.Apps[i];
            UNAS.LoadJS("/apps/" + app.appDir + "/" + app.appJS + "?v=" + app.appVersion, function() {
              UNAS.isAppCountAllLoad--; //when this callback is called, it means App JS loaded
              if (UNAS.isAppCountAllLoad <= 0) { //now all apps loaded
                UNAS.isInit = true;
                UNAS.renderMenuPanel();
                if (callback && typeof callback === "function") {
                  callback();
                }
              }
            }, false); //load desktop after the last App and fire ChangeLanguage event
          }
        }
      }
    },
    onFailure: function() {
      alert(UNAS._('Loading Apps failed!'));
    }
  });
  req.send();
}

/**
 *Load app by app name
 *@param {String} appName - The App name to be loaded
 */
UNAS.LoadApp = function(appName, callback) {
  // Use Mootools ajax to get apps
  var req = new Request({
    url: '/api/php/UNASJS.php',
    method: 'POST',
    data: {
      'appName': appName,
      'action': 'GetApp'
    },
    onRequest: function() {},
    onSuccess: function(responseText) {
      var app = JSON.parse(responseText);
      if (app != null) {
        UNAS.Apps.push(app);
        UNAS.LoadJS("/apps/" + app.appDir + "/" + app.appJS + "?v=" + app.appVersion, callback);
      }
    },
    onFailure: function() {
      alert(UNAS._('Loading App: ') + appName + UNAS._(' failed!'));
    }
  });
  req.send();
}
/**
 *Load app by app.js path
 *@param {String} appPath - App's path
 *@return
 */
UNAS.LoadAppByPath = function(appPath) {
  // Use Mootools ajax to get apps
  var req = new Request({
    url: '/api/php/UNASJS.php',
    method: 'POST',
    data: {
      'appPath': appPath,
      'action': 'GetAppByPath'
    },
    onRequest: function() {},
    onSuccess: function(responseText) {
      var app = JSON.parse(responseText);
      if (app != null) {
        UNAS.Apps.push(app);
        UNAS.LoadJS("/apps/" + app.appDir + "/" + app.appJS + "?v=" + app.appVersion);
      }
    },
    onFailure: function() {
      alert('Loading App: ' + appPath + ' failed!');
    }
  });
  req.send();
}
/**
 *Remove App by app Name
 *@param {String} appName - The name of app to be removed
 *@return
 */
UNAS.RemoveApp = function(appName) {
  // Use Mootools ajax to remove apps
  var req = new Request({
    url: '/api/php/UNASJS.php',
    method: 'POST',
    data: {
      'appName': appName,
      'action': 'RemoveApp'
    },
    onRequest: function() {},
    onSuccess: function(responseText) {
      var parentDirectory = JSON.parse(responseText);
      for (var i = 0; i < UNAS.Apps.length; i++) {
        if (UNAS.Apps[i].appDir == parentDirectory) {
          //found it
          for (var j = 0; j < UNAS.Apps[i].appNodes.length; j++) {
            if (UNAS.Apps[i].appNodes[j] != null && UNAS.Apps[i].appNodes[j].parentNode != null)
              UNAS.Apps[i].appNodes[j].parentNode.removeChild(UNAS.Apps[i].appNodes[j]);
          }
          return;
        }
      }
    },
    onFailure: function() {
      alert('Remove App: ' + appName + ' failed!');
    }
  });
  req.send();
}
/**
 * Start app
 * @param {String} appName - the name of app
 * @param {Ojbect} appArgs - the Arguments to run the app
 * @return
 */
UNAS.StartApp = function(appName, appArgs) {
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == appName) {
      //found it,run ...
      if (typeof UNAS.Apps[i].appStartFunc !== 'undefined') {
        UNAS.Apps[i].appStartFunc(appArgs);
      }
    }
  }
}

/**
 * Create App, Before loading app,an UNAS.AppInfo item will be created and pushed into UNAS.Apps[],
 * App Calls this function when it is loaded to update the name and start fucntion/arguments.
 * appStartFunc can take arguments, just list in this function's argument list after 'appStartFucn'.
 *@param {String} appName - App Name
 *@param {String} appIcon - App Icon, suppport favicon or single icon >= 32x32
 *@param {Function} appStartFunction - Start function for the app
 *@param {Function} appUninstallFunc - Uninstall function for the app
 *@param {Ojbect} appState - The state of app,state and opened window id {'state':'active','window':'widow_id'}
 *@param {Array} appStartFuncArgs - The args of app start function
 *@param {Array} appUninstallFuncArgs - The args of app uninstall function
 *@param
 */
UNAS.CreateApp = function(appName, appIcon, appStartFunc, appUninstallFunc, appState, appStartFuncArgs, appUninstallFuncArgs) {
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == appName) {
      //found it, settings ...
      if (typeof appIcon !== 'undefined')
        UNAS.Apps[i].appIcon = appIcon; //update app Icon, support favicon or single icon >= 32x32
      UNAS.Apps[i].appStartFunc = appStartFunc; //start function
      if (typeof appUninstallFunc !== 'undefined')
        UNAS.Apps[i].appUninstallFunc = appUninstallFunc;
      if (typeof appState !== 'undefined')
        UNAS.Apps[i].appState = appState;
      else
        UNAS.Apps[i].appState = {
          'state': 'inactive',
          'window': ''
        };
      if (typeof appStartFuncArgs !== 'undefined')
        UNAS.Apps[i].appStartFuncArgs = appStartFuncArgs;
      if (typeof appUninstallFuncArgs !== 'undefined')
        UNAS.Apps[i].appUninstallFuncArgs = appUninstallFuncArgs;
    }
  }
}
/**
 * Get App Information form Unas.Apps
 * @param {String} appName The name of app
 * return {Array|null} Array of Apps info or null if not found
 */
UNAS.GetAppInfo = function(appName) {
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == appName) {
      return UNAS.Apps[i];
    }
  }
  return null;
}
/**
 * Remove App from UNAS.Apps
 * @param {String} AppName - The name of app to be removed
 */
UNAS.RemoveAppFromApps = function(AppName) {

  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == AppName) {
      UNAS.Apps.splice(i, 1);
      break;
    }
  }
  return true;
}
/**
 *UNAS Load JS to <head> or <body>(if no <head>), with callback onloaded
 *@function Dynamically load JS files
 *@param {String} jsFile - JS file name to be loaded
 *@param {Function} callback - Callback function after loaded
 *@param {boolean} async - Asynchronouse or not, default is true
 */
UNAS.LoadJS = function(jsFile, callback, async) {
  if (typeof async ==='undefined') async = true; //default is true.
  if (typeof jsFile === 'undefined') {
    return false;
  }
  var script = document.createElement('script');
  script.type = 'text/javascript';
  script.async = async;
  if (script.readyState) {
    script.onreadystatechange = function() {
      if (script.readyState == "loaded" || script.readyState == "complete") {
        script.onreadystatechange = null;
        if (callback && typeof callback === "function") {
          callback();
        }
      }
    };
  } else {
    script.onload = function() {
      if (callback && typeof callback === "function") {
        callback();
      }
    };
  }
  script.src = jsFile;
  (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(script);
}
// 加载图片
UNAS.LoadImg = function(imgFile, callback, async) {
  if (typeof async ==='undefined') async = true; //default is true.
  if (typeof imgFile === 'undefined') {
    return false;
  }
  var img = document.createElement('img');
  img.style.display = 'none';
  img.id = 'defaultDesktopBgImg'
  img.async = async;
  if (img.readyState) {
    img.onreadystatechange = function() {
      if (img.readyState == "loaded" || img.readyState == "complete") {
        img.onreadystatechange = null;
        if (callback && typeof callback === "function") {
          callback();
        }
      }
    };
  } else {
    img.onload = function() {
      if (callback && typeof callback === "function") {
        callback();
      }
    };
  }
  img.src = imgFile;
  (document.getElementsByTagName('body')[0]).appendChild(img);
}
/**
 *UNAS Load CSS, CSS loaded to <head>
 *@function Dynamically load JS files
 *@param {String} cssFile - CSS file name to be loaded
 *@param {Function} callback - Callback function after loaded
 *@return {Element} css element
 */
UNAS.LoadCSS = function(cssFile, callback) {
  if (typeof cssFile === 'undefined' || typeof document.getElementsByTagName("head")[0] === 'undefined') {
    return false;
  }
  cssFile = cssFile + '?v=6.1.43&';

  var fileref = document.createElement("link");
  fileref.setAttribute("rel", "stylesheet");
  fileref.setAttribute("type", "text/css");
  fileref.setAttribute("href", cssFile);
  if (fileref.readyState) {
    fileref.onreadystatechange = function() {
      if (fileref.readyState == "loaded" || fileref.readyState == "complete") {
        fileref.onreadystatechange = null;
        if (callback && typeof callback === "function") {
          callback();
        }
      }
    };
  } else {
    fileref.onload = function() {
      if (callback && typeof callback === "function") {
        callback();
      }
    };
  }
  if (typeof fileref != "undefined")
    document.getElementsByTagName("head")[0].appendChild(fileref);
  return fileref; //return the element.
}
/**
 *UNAS create JS or CSS node
 *@param {String} filename - CSS/JS file to be loaded
 *@param {String} filetype - 'css' or 'js' for file type
 *@returns {Element} CSS/JS element
 */
UNAS.CreateJSCSSFile = function(filename, filetype) {
  if (filetype == "js") { //if filename is a external JavaScript file
    var fileref = document.createElement('script')
    fileref.setAttribute("type", "text/javascript")
    fileref.setAttribute("src", filename)
  } else if (filetype == "css") { //if filename is an external CSS file
    var fileref = document.createElement("link")
    fileref.setAttribute("rel", "stylesheet")
    fileref.setAttribute("type", "text/css")
    fileref.setAttribute("href", filename)
  }
  return fileref
}
/**
 *UNAS Replace JS or CSS
 *@function Dynamically load JS files
 *@param {String} oldfilename - CSS/JS file to be replaced
 *@param {String} newfilename - CSS/JS file to be loaded
 *@param {String} filetype - 'css' or 'js' for file type
 */
UNAS.ReplaceJSCSSFile = function(oldfilename, newfilename, filetype) {
  var targetelement = (filetype == "js") ? "script" : (filetype == "css") ? "link" : "none" //determine element type to create nodelist using
  var targetattr = (filetype == "js") ? "src" : (filetype == "css") ? "href" : "none" //determine corresponding attribute to test for
  var allsuspects = document.getElementsByTagName(targetelement)
  for (var i = allsuspects.length; i >= 0; i--) { //search backwards within nodelist for matching elements to remove
    if (allsuspects[i] && allsuspects[i].getAttribute(targetattr) != null && allsuspects[i].getAttribute(targetattr).indexOf(oldfilename) != -1) {
      var newelement = UNAS.CreateJSCSSFile(newfilename, filetype)
      allsuspects[i].parentNode.replaceChild(newelement, allsuspects[i])
    }
  }
}
/**
 *UNAS Set Wallpaper
 *@param {String} filename - File name of the wallpaper (full path)
 *@param {String} fit - Wallpaper fit
 */
UNAS.SetWallpaper = function(filename, fit) {

  filename = filename || '/desktop/themes/default/wallpaper.png';
  fit = fit || 'fill';
  filename += "?v=6.1.43&";
  jQuery('.unas-desktop').css('background-image', 'url(' + filename + ')');
  jQuery('.unas-desktop').css({
    'background-size': 'cover',
    'background-repeat': 'no-repeat'
  });
  jQuery('#menuAppsContainer').append('<style>.menu-apps::before{background-image:url("' + filename + '")}</style>').children('style').not(':last-child').remove();
  jQuery('.unas-desktop').css('filter', 'progid:DXImageTransform.Microsoft.AlphaImageLoader(src="' + filename + '", sizingMethod="scale")\9;');
  UNAS.Desktop.Wallpaper = filename; //keep this info
}
/**
 * 创建版本信息窗口
 */
UNAS.CreateVersionAppWindow = function() {
  new MUI.Modal({
    id: 'VersionAppWindow',
    title: UNAS._('Version'),
    loadMethod: 'xhr',
    type: 'modal2',
    width: 680,
    height: 250,
    maximizable: false,
    resizable: true,
    scrollbars: false,
    modalOverlayClose: false,
    contentURL: '/desktop/html/version.html'
  });
}

/**
 *UNAS Menu
 * @param {String} parent the ID of the parent menu
 * @param {String} id The id of the menu
 * @param {String} icon the url of the icon for the menu
 * @param {String} text the menu text
 * @param {String} status
 * @param {function} func the click event handler
 * @returns {Element} The menu item element. or null if not successful
 */
UNAS.Menu = function(parent, id, icon, text, status, func) {
  if (typeof parent === 'undefined' || parent === '') return;
  var m = document.getElementById(parent); //find parent menu in the system menu frame
  if (!m) return;
  var mi = document.createElement("LI"); //create menu item
  mi.className = 'icon_pw'; //need to work on this name!!!!
  mi.id = id + 'Menu';
  var menuIMG = document.createElement("IMG"); //create image for menu item
  menuIMG.style.width = '16px';
  menuIMG.style.height = '16px';
  menuIMG.src = icon;
  mi.appendChild(menuIMG);
  var menuA = document.createElement('span'); //carete <a> for menu item
  var menuTextNode = document.createTextNode(text); //create text node for <a> (display text in menu)
  menuA.appendChild(menuTextNode);
  mi.appendChild(menuA);
  if (m) {
    if (!m.hasChildNodes()) m.style.visibility = 'visible';
    m.appendChild(mi); //insert menu item into the menu frame
    //Add menu click event handler
    //menuA.addEventListener();
    if (func && typeof func === "function") {
      // addEventSimple(menuA, 'click', func);
      addEventSimple(mi, 'click', func);
    }
    return mi;
  }
  return null;
}
/**
 *UNAS Start Menu
 * @param {String} parent the ID of the parent menu
 * @param {String} id The id of the menu
 * @param {String} text the menu text
 * @param {function} func the click event handler
 * @returns {Element} The menu item element. or null if not successful
 */
UNAS.StartMenu = function(parent, id, text, func, pre, iconSrc, hasChildren) {
  if (typeof parent === 'undefined' || parent === '') return;
  var mi = document.createElement("li"); //create menu item
  mi.id = id + 'Menu';
  var img = document.createElement('img');
  img.src = iconSrc;
  var span = document.createElement('span');
  span.innerHTML = text;
  mi.appendChild(img);
  mi.appendChild(span);
  var m = document.getElementById(parent); //find parent menu in the system menu frame
  if (m) {
    // if (!m.hasChildNodes()) m.style.visibility = 'visible';
    if (typeof pre === 'undefined' || pre === '') {
      m.appendChild(mi); //insert menu item into the menu frame
    } else {
      if (m.firstChild) {
        m.insertBefore(mi, m.firstChild);
      } else {
        m.appendChild(mi);
      }
    }
    //Add menu click event handler
    //img.addEventListener();
    if (id != 'About') {
      if (func && typeof func === "function") {
        addEventSimple(mi, 'click', func, function() {
          if (jQuery("#desktopHiddenModal").length) {
            jQuery("#desktopHiddenModal").trigger('mousedown');
          }
        });
      }
    } else {
      var aboutI = document.createElement('i');
      // aboutI.innerHTML = '&#9658';
      mi.className = 'li-about toggleDisplayHoverBlock';
      mi.appendChild(aboutI);
      var aboutUl = document.createElement('ul');
      aboutUl.id = 'aboutMeunUl';
      mi.appendChild(aboutUl);
      //二级弹出块监听函数
      jQuery('#' + mi.id).hover(function(e) {
        jQuery(this).siblings().removeClass('active').children('ul').css('display', 'none');
        jQuery(this).children('ul').css('display', 'block');
      }, function(e) {
        var that = this;
        setTimeout(function() {
          jQuery(that).children('ul').css('display', 'none');
        }, 200);
      });
      jQuery('#' + aboutUl.id).hover(function(e) {
        e.stopPropagation();
        jQuery(this).parent().addClass('active');
      }, function(e) {
        e.stopPropagation();
        // jQuery(this).parent().removeClass('active');
      });
      // debugger;
      UNAS.StartMenu('UNAS_Top_Menu', 'officialWeb', 'Official Web', function() {
        // window.open(unas_company_website);
        if (UNAS.CurrentLanguage == "简体中文") {
          window.open("http://www.u-nas.cn");
        } else {
          window.open(unas_company_website);
        }
      }, '', '/desktop/images/ic-systeminfo/ic-officialwebsite.png?v=6.1.43&');
      UNAS.StartMenu('UNAS_Top_Menu', 'version', 'Version', function() {
        UNAS.CreateVersionAppWindow();
      }, 'pre', '/desktop/images/ic-systeminfo/ic-version.png?v=6.1.43&');
    }

    return mi;
  }
  return null;
}
/**
 *UNAS User Menu
 * @param {String} parent the ID of the parent menu
 * @param {String} id The id of the menu
 * @param {String} text the menu text
 * @param {function} func the click event handler
 * @returns {Element} The menu item element. or null if not successful
 */
UNAS.UserMenu = function(parent, id, text, func, iconSrc) {
  if (typeof parent === 'undefined' || parent === '') return;
  var mi = document.createElement("div"); //create menu item
  mi.id = id + 'Menu';
  var img = document.createElement('img');
  img.src = iconSrc;
  var menuP = document.createElement('p'); //carete <a> for menu item
  var menuTextNode = document.createTextNode(text); //create text node for <a> (display text in menu)
  menuP.appendChild(menuTextNode);
  mi.appendChild(img);
  mi.appendChild(menuP);
  var m = document.getElementById(parent); //find parent menu in the system menu frame
  if (m) {
    if (!m.hasChildNodes()) m.style.visibility = 'visible';
    m.insertBefore(mi, document.getElementById("User_Menu_Logout"));
    //Add menu click event handler
    //menuP.addEventListener();
    if (func && typeof func === "function") {
      addEventSimple(mi, 'click', func, function() {
        if (jQuery("#desktopHiddenModal").length) {
          jQuery("#desktopHiddenModal").trigger('mousedown');
        }
      });
    }
    return mi;
  }
  return null;
}
/**
 *Get Menu element by id
 *@param {String} id - id of the menu
 *@returns {Element|null} Menu element or null
 */
UNAS.GetMenu = function(id) {
  if (typeof id !== 'undefined') {
    return document.getElementById(id + 'Menu');
  }
  return null;
}
/**
 *Set Menu Title
 *@param {String} id - id of the menu
 *@param {String} t - title of the menu
 */
UNAS.SetMenuTitle = function(id, t) {
  if (typeof id !== 'undefined') {
    var m = UNAS.GetMenu(id);
    if (m != null) {
      var mt = UNAS.TextNodesUnder(m);
      if (mt.length > 0) {
        mt[0].nodeValue = t;
        m.attributes.title && (m.attributes.title.value = t);
      }
    }
  }
}

/**
 *Set Menu Position
 *@param {String} id - id of the menu
 *@param {Integer} pos - position of the menu
 */
UNAS.SetMenuPosition = function(id, pos) {
  if (typeof id !== 'undefined') {
    var m = UNAS.GetMenu(id);
    if (m != null && m.parentElement != null) {
      m.parent.removeChild
      if (pos > m.parentElement.length || typeof pos === 'undefined') {
        m.parentElement.appendChild(m); //append if out of range
      } else { //insert..
        m.parentElement.insertBefore(m, m.parentElement.childNodes[pos]);
      }
    }
  }
}
/**
 *Remove Menu element by id
 *@param {String} id - id of Menu element to be remove
 *@return
 */
UNAS.RemoveMenu = function(id) {
  var appMenu = UNAS.GetMenu(id);
  if (appMenu != null) {
    appMenu.parentNode.removeChild(appMenu);
  }
}

/**
 *UNAS Display/Hide an Menu
 *@function Display/Hide Menu
 *@param {String} id - Menu id
 *@param {Boolean} show - Show/Hide Menu, true will show menu
 */
UNAS.ShowMenu = function(id, show) {
  show = show || true; //default is true
  var el = document.getElementById(id);
  if (el) {
    el.style.visibility = 'hidden';
    if (el.hasChildNodes()) {
      if (show) {
        el.style.visibility = 'visible';
      }
    }
  }
}
/**
 *Get Shortcut element by id
 *@param {String} id-  Shortcut id
 *@returns {Element|null} Shortcut element or null
 */
UNAS.GetShortcut = function(id) {
  if (typeof id !== 'undefined') {
    return document.getElementById(id + 'DesktopShortcut');
  }
  return null;
}
/**
 *Set Shortcut Title
 *@param {String} id - id of the Shortcut
 *@param {String} t - title of the Shortcut
 */
UNAS.SetShortcutTitle = function(id, t) {
  if (typeof id !== 'undefined') {
    var s = UNAS.GetShortcut(id);
    if (s != null) {
      var st = UNAS.TextNodesUnder(s);
      if (st.length > 0) {
        st[0].nodeValue = t;
      }
    }
  }
}
/**
 *Remove Shortcut element by id
 *@param {String} id - of shortcut element to be remove
 *@return
 */
UNAS.RemoveShortcut = function(id) {
  var appShortcut = UNAS.GetShortcut(id);
  if (appShortcut != null) {
    appShortcut.parentNode.removeChild(appShortcut);
  }
}

/**
 *UNAS  Shortcut to Desktop.
 *@function Dynamically add app shortcut
 *@param {String} id - Shortcut id
 *@param {String} icon - icon file path
 *@param {String} test - Shortcut text
 *@param {function} func - double click event handler, additonal events can be added by addEventSimple(obj,evt,fn)
 *@return {Element} the shortcut item element or null if not successful
 */
UNAS.DesktopShortcut = function(id, icon, text, func) {
  var shortcutA = document.createElement('a'); //carete <a> for menu item
  shortcutA.className = 'abs unas_icon menu-target';
  shortcutA.id = id + 'DesktopShortcut';
  shortcutA.style.width = UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutWidth;
  shortcutA.style.height = UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutHeight;

  var pos = UNAS.GetNextShortcutPosition();
  UNAS.SetShortcutPosition(shortcutA, pos.x, pos.y); //set posititon

  var shortcutIMG = document.createElement("IMG"); //create image for shortcut item
  shortcutIMG.style.width = UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutIconSize + 'px';
  shortcutIMG.style.height = UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutIconSize + 'px';
  shortcutIMG.src = icon;
  shortcutA.appendChild(shortcutIMG);

  var shortcutSPAN = document.createElement("SPAN");
  shortcutSPAN.style.padding = "0px 3px 0 3px";
  shortcutSPAN.style.display = "block";
  var shortcutTextNode = document.createTextNode(text); //create text node for <a> (display text in menu)
  shortcutSPAN.appendChild(shortcutTextNode);
  shortcutA.appendChild(shortcutSPAN);

  var d = UNAS.GetCurrentVirtualDesktopDIV(); //find Desktop div
  d.appendChild(shortcutA); //insert shortcut item into the Deskotp

  //Add shortcut double click event handler
  var clickFunc = null;
  if (func && typeof func === "function") {
    //addEventSimple(shortcutA, "dblclick", func); //double click to run shortcut
    addEventSimple(shortcutA, "click", func); //click to run shortcut
    clickFunc = func;
  }

  //dragDrop.initElement(shortcutA);    //shortcut dragdrop
  shortcutA.draggable = "true";
  var dragSrcID = "";
  shortcutA.ondragstart = function(evt) {
    // if(evt.currentTarget.parentElement == jQuery("#desktopNavbar")[0]){
    UNAS.StartPosX = evt.currentTarget.offsetLeft; //在顶部栏内部拖动时，记录初始left值
    UNAS.StartPosY = evt.currentTarget.offsetTop;
    UNAS.StartParentEle = evt.currentTarget.parentElement;
    // }
    if (UNAS.ElementCanDrag) {
      evt.dataTransfer.setData("text/plain", shortcutA.id);
    }
    dragSrcID = shortcutA.id;
    evt.dataTransfer.setDragImage(shortcutA, shortcutA.offsetWidth / 2, shortcutA.offsetHeight / 2);
  }

  shortcutA.ondragover = function(evt) { //drag over handler
    evt.preventDefault();
    var pID = evt.dataTransfer.getData("text/plain");
    //in chrome, evt.dataTransfer only usable in drop, not here!!
    if (shortcutA.id == pID) return; //on itself,
  }
  shortcutA.ondragenter = function(evt) { //drag enter handler
    // console.log("enter",evt)
  }
  shortcutA.ondragleave = function(evt) { //drag leave handler
    // console.log("leave",evt)
  }
  shortcutA.ondragend = function(evt) { //drag end
    if (UNAS.ElementCanDrag && UNAS.IsDropTarget) {
      if (UNAS.ElementSortType == 'gridAlign' && UNAS.DesktopDropTarget == "pageWrapper") {
        var dtpInfo = UNAS.GetDesktopPositionInfo("pageWrapper");

        var intArr = [],
          positionArr = [];
        positionArr.push(evt.currentTarget.offsetLeft + evt.currentTarget.offsetWidth / 2) //x position
        positionArr.push(evt.currentTarget.offsetTop + evt.currentTarget.offsetHeight / 2); //y position
        intArr.push(Math.ceil(positionArr[1] / dtpInfo.IconY)); //屏幕占比y位置,第几行
        intArr.push(Math.ceil(positionArr[0] / UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutWidth)); //屏幕占比x位置
        //防止计算出相同位置图标遮盖，有相同位置，则返回原来位置
        for (var key in UNAS.CMGridAlignls) {
          if (UNAS.CMGridAlignls[key] != (dtpInfo.IconN * (intArr[1] - 1) + intArr[0])) {
            continue;
          } else {
            jQuery(evt.currentTarget).css({
              "top": (UNAS.CMGridAlignls[evt.currentTarget.id] % dtpInfo.IconN) > 0 ? ((UNAS.CMGridAlignls[evt.currentTarget.id] % dtpInfo.IconN) - 1) * dtpInfo.IconY : (dtpInfo.IconN - 1) * dtpInfo.IconY + "px",
              "left": ((Math.ceil(UNAS.CMGridAlignls[evt.currentTarget.id] / dtpInfo.IconN) - 1)) * UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutWidth + "px"
            })
            return;
          }
        }
        UNAS.CMGridAlignls[evt.currentTarget.id] = dtpInfo.IconN * (intArr[1] - 1) + intArr[0];
        jQuery(evt.currentTarget).css({
          "top": (UNAS.CMGridAlignls[evt.currentTarget.id] % dtpInfo.IconN) > 0 ? ((UNAS.CMGridAlignls[evt.currentTarget.id] % dtpInfo.IconN) - 1) * dtpInfo.IconY : (dtpInfo.IconN - 1) * dtpInfo.IconY + "px",
          "left": ((Math.ceil(UNAS.CMGridAlignls[evt.currentTarget.id] / dtpInfo.IconN) - 1)) * UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutWidth + "px"
        })
      }
      if (UNAS.DesktopDropTarget == "desktopNavbar") {
        for (var key in UNAS.CMGridAlignls) {
          if (evt.currentTarget.id == key) {
            delete UNAS.CMGridAlignls[key];
            break;
          }
        }
        var avaiableW = parseInt(jQuery("#desktopNavbar").outerWidth()); //可放置图标的宽度
        var n = parseInt(avaiableW / (evt.currentTarget.offsetWidth)); //能放多少个图标
        var nW = avaiableW / n; //一个图标平均占多宽
        var siteN = Math.ceil((evt.currentTarget.offsetLeft + evt.currentTarget.offsetWidth - 86) / nW); //占第几个位置
        var CW = 86; //左边开始菜单宽度
        var navbarA = jQuery("#navbarShortcutContainer>a.abs.unas_icon.menu-target");
        UNAS.NavbarSiteNum = {};
        if (navbarA.length > 1) {
          for (var i = 0; i < navbarA.length; i++) {
            var aSite = Math.ceil(((parseInt(jQuery(navbarA[i]).css("left")) + parseInt(jQuery(navbarA[i]).outerWidth()) / 2) - 86) / nW);
            if (navbarA[i].id != evt.currentTarget.id) {
              UNAS.NavbarSiteNum[aSite] = navbarA[i].id;
            } else {
              if (UNAS.NavbarSiteNum[aSite] == evt.currentTarget.id) {
                delete UNAS.NavbarSiteNum[aSite];
              }
            }
          }
          while (siteN) {
            if (UNAS.NavbarSiteNum[siteN]) {
              siteN++;
            } else {
              break;
            }
          }
        }
        CW = (siteN - 1) * nW;
        jQuery(evt.currentTarget).css("left", CW + 86);
      }
      // console.log(UNAS.CMGridAlignls)
      UNAS.SaveUserDesktop();
    }
    UNAS.IsDropTarget = false;
  }
  shortcutA.ondrop = function(evt) { //drop handler
    evt.preventDefault();
    evt.stopPropagation();
    var pID = evt.dataTransfer.getData("text/plain");
    if (shortcutA.id == pID) return; //on itself
    //by default, drop to shortcut does not move the source, call click function with the dataTransfer as arg?
    if (clickFunc != null) {
      clickFunc(evt.dataTransfer); //App can determine what to do
    }
  }
  // shortcutA.onmouseover = function(e){ //hover handler
  //     jQuery(e.target).addClass("hover").siblings().removeClass("hover");
  // }
  // shortcutA.onmousedown = function(e){ //mousedown handler
  //     e.stopPropagation();
  // }
  UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].Shortcuts.push(shortcutA); //add it to the list
  // 安装或者更新时调用，需判断是否初始化完成和桌面是否加载完成
  if (UNAS.isInit && UNAS.isDesktopReady) {
    if (!UNAS.CMGridAlignls[shortcutA.id]) { //值不存在说明是新安装，存在说明是更新
      UNAS.ResortAfterInstall(shortcutA.id);
    } else {
      UNAS.ResortAfterInstall(shortcutA.id, "upgrade");
    }
  }
  return shortcutA;
}
/**
 *Get Desktop Shortcut by id
 *@param {String} id - id of the shortcut
 *@return {Element|null} the shortcut element or null
 */
UNAS.GetDesktopShortcut = function(id) {
  if (typeof id !== 'undefined') {
    return document.getElementById(id + 'DesktopShortcut');
  }
  return null;
}
/**
 *Remove Desktop Shortcut by id
 * @param {String} id - id of the shortcut to be removed
 */
UNAS.RemoveDesktopShortcut = function(id) {
  if (typeof id !== 'undefined') {
    var elem = UNAS.GetDesktopShortcut(id);
    if (elem != null) {
      elem.parentNode.removeChild(elem);
    }
  }
}

/**
 *UNAS Get next shortcut position
 *@function Get next shortcut position
 *@returns {x,y} the next avaiable position of shortcut
 */
UNAS.GetNextShortcutPosition = function() {
  var vd = UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop];
  if (vd) {
    var x = vd.ShortcutLeft;
    var y = vd.ShortcutTop;
    vd.ShortcutTop += vd.ShortcutDeltaY;
    if (vd.ShortcutTop > vd.Top0 + vd.ShortcutDeltaY * vd.ShortcutIconsPerColumn) {
      vd.ShortcutTop = vd.Top0;
      vd.ShortcutLeft += vd.ShortcutDeltaX;
      if (vd.ShortcutLeft > screen.width) {
        vd.ShortcutLeft = vd.Left0; //cannot move out of screen.
      }
    }
  }
  return {
    'x': x,
    'y': y
  };
}
/**
 *UNAS Set Shortcut Position
 *@function Display/Hide Menu
 *@param {number} x - x position, donot inlude 'px'
 *@param {number} y - y position, donot inlude 'px'
 */
UNAS.SetShortcutPosition = function(shortcut, x, y) {
  if (typeof x === 'undefined' || typeof y === 'undefined') return;
  shortcut.style.left = x + 'px';
  shortcut.style.top = y + 'px';
}

/**
 * Register App info to UNAS.AppGroups.
 * Example: UNAS.RegisterToAppGroup('Account', 'ControlPanel',{Icon:, Type：, Location:, Icon: Url:},{{event:'dbclick', eventHandler: func},{event:'click', eventHandler: func}});
 * @param {String} appName - Name of the App (used in UNAS.CreateApp)
 * @param {String} AppGroupName - the appgroup name witch app belonged to
 * @param {Object} options - options passed in {Icon:, Type: , Location: , Icon:, Url:}
 * @param {Object} events - the events {{event: , eventHandler:},...}
 */
UNAS.RegisterToAppGroup = function(AppName, AppGroupName, options, events) {
  if (typeof(UNAS.AppGroups[AppGroupName]) == 'undefined') {
    UNAS.AppGroups[AppGroupName] = [{
      'AppName': AppName,
      'AppGroupName': AppGroupName,
      'options': options,
      'events': events
    }];
  } else {
    UNAS.AppGroups[AppGroupName].push({
      'AppName': AppName,
      'AppGroupName': AppGroupName,
      'options': options,
      'events': events
    });
  }
  UNAS.AppGroupsAppNames[AppName] = AppName;
}
UNAS.SetAppState = function(appName, AppState) {
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == appName) {
      if (typeof AppState !== 'undefined')
        UNAS.Apps[i].appState['state'] = AppState;
    }
  }
}
UNAS.SetAppOpenedWindow = function(appName, WindowID) {
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (UNAS.Apps[i].appName == appName) {
      if (typeof WindowID !== 'undefined')
        UNAS.Apps[i].appState['window'] = WindowID;
    }
  }
}
UNAS.GetAppState = function(AppName) {
  var appstate = UNAS.GetAppInfo(AppName);
  if (appstate == null) {
    return {
      'state': 'inactive'
    };
  }
  return appstate['appState'];
}
UNAS.CheckAppState = function(AppName) {
  var appstate = UNAS.GetAppState(AppName);
  if (appstate['state'] == 'inactive') {
    return false;
  } else {
    if (!UNAS.ShowWindow(appstate['window'])) {
      return false;
    }
    return true;
  }
}
UNAS.ShowWindow = function(window_id) {
  var instance = UNAS.GetWindow(window_id);
  if (instance == null)
    return false;
  if (instance.isMinimized) {
    MUI.Dock.restoreMinimized($(window_id));
  } else if (instance.isCollapsed) {
    MUI.collapseToggle($(window_id));
    setTimeout(MUI.focusWindow.pass($(window_id), $(window_id)), 10);
  } else {
    var coordinates = document.getCoordinates();
    if ($(window_id).getStyle('left').toInt() > coordinates.width || $(window_id).getStyle('top').toInt() > coordinates.height) {
      MUI.centerWindow($(window_id));
    }
    setTimeout(MUI.focusWindow.pass($(window_id), $(window_id)), 10);
    if (MUI.options.standardEffects == true) {
      $(window_id).shake();
    }
  }
  return true;
}
/**
 * Remove element from app group
 *@param {String} appName - Name of the App (used in UNAS.CreateApp)
 * @param {String} AppGroupName - the AppGroup name to be removed from
 */
UNAS.RemoveAppFromGroups = function(AppName, AppGroupName) {
  if (typeof(UNAS.AppGroups[AppGroupName]) == 'undefined')
    return false;
  else {
    for (var i = 0; i < UNAS.AppGroups[AppGroupName].length; i++) {
      if (UNAS.AppGroups[AppGroupName][i].AppName == AppName) {
        UNAS.AppGroups[AppGroupName].splice(i, 1);
        break;
      }
    }
  }
}
/**
 * UNAS Custom Event
 * Add Event Handler. Support unlimited event type
 *  UNAS.Event.addEvent("ChangeLanguage", function() {
 *      alert("Event fired");
 *  });
 * Trigger:
 *  UNAS.Event.fireEvent("ChangeLanguage");
 **/
UNAS.Event = {
  _listeners: {},
  //Add event hangler
  addEvent: function(type, fn) {
    if (typeof this._listeners[type] === "undefined") {
      this._listeners[type] = [];
    }
    if (typeof fn === "function") {
      //insert a listener only it is not in the array.
      if (this._listeners[type].indexOf(fn) > -1) {} else {
        this._listeners[type].push(fn);
      }
    }
    return this;
  },
  //Trigger
  fireEvent: function(type) {
    var arrayEvent = this._listeners[type];
    if (arrayEvent instanceof Array) {
      for (var i = 0, length = arrayEvent.length; i < length; i += 1) {
        if (typeof arrayEvent[i] !== 'undefined' && typeof arrayEvent[i] === "function") {
          try {
            arrayEvent[i]({
              type: type
            });
          } catch (e) {
            //event call not successful
          }
        }
      }
    }
    return this;
  },
  //Remove Event Handler
  removeEvent: function(type, fn) {
    var arrayEvent = this._listeners[type];
    if (typeof type === "string" && arrayEvent instanceof Array) {
      if (typeof fn === "function") {
        //remove, based on type and function
        for (var i = 0, length = arrayEvent.length; i < length; i += 1) {
          if (arrayEvent[i] === fn) {
            this._listeners[type].splice(i, 1);
            break;
          }
        }
      } else {
        // remove all functions associated with the type.
        delete this._listeners[type];
      }
    }
    return this;
  }
};
/**
 * get desktopIcon should take height,width,cloumns,
 * @param id
 * @return {object}
 * Sw:screen width,Sh:screen height
 * IconN:how many icons can put in each column
 * IconY:the average height of an icon
 */
UNAS.GetDesktopPositionInfo = function(id) {
  var dkptInfo = {};
  //假设图标宽高分别为x:80,y:100,图标总数为m;
  dkptInfo.Sw = jQuery(document.getElementById(id)).width();
  dkptInfo.Sh = jQuery(document.getElementById(id)).height() - 10;
  //每一列最多能放多少个图标
  dkptInfo.IconN = Math.floor(dkptInfo.Sh / UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutHeight);
  //每一行最多能放多少个图标
  dkptInfo.IconM = Math.floor(dkptInfo.Sw / UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutWidth);
  //平均一个图标占多少高度
  dkptInfo.IconY = dkptInfo.Sh / dkptInfo.IconN;
  //有多少列 m/IconN
  return dkptInfo;
}

/**
 * create shortcut with contextMenu in apps
 * @param id
 */
UNAS.CreateShortcutWithContextMenu = function(id) {
  var tId = id.replace(/DesktopShortcut/g, "");
  for (var i = 0; i < UNAS.Apps.length; i++) {
    if (tId == UNAS.Apps[i].appName) {
      if (document.getElementById(id + "DesktopShortcut") != null) {
        return;
      }
      UNAS.DesktopShortcut(tId, UNAS.Apps[i].appIcon, UNAS._(UNAS.Apps[i].appName), UNAS.Apps[i].appStartFunc);
      break;
    }
  }
  var dstpInfo = UNAS.GetDesktopPositionInfo("pageWrapper");
  if (UNAS.ElementSortType == 'autoArrange') {
    //自动排序时，在有序对象的末尾增加一位
    var c = [];
    for (var key in UNAS.CMGridAlignls) {
      c.push(UNAS.CMGridAlignls[key]);
    }
    c.sort(function(a, b) {
      return a - b
    });
    UNAS.CMGridAlignls[id + "DesktopShortcut"] = c.length + 1;
    UNAS.__autoArrangeFun(dstpInfo);
  } else if (UNAS.ElementSortType == 'gridAlign') {
    //网格排序时，在第一个空白位置放置新增的图标
    var d = {},
      e = 1;
    for (var key in UNAS.CMGridAlignls) {
      d[UNAS.CMGridAlignls[key]] = key;
    }
    while (e) {
      if (d[e]) {
        e++;
      } else {
        d[e] = id + "DesktopShortcut";
        break;
      }
    }
    UNAS.CMGridAlignls = {};
    for (var key in d) {
      UNAS.CMGridAlignls[d[key]] = Number(key);
    }
    UNAS.__gridAlignFun(dstpInfo);
  }
  UNAS.CSWCMtrans(id)
}
/**
 * 桌面右键自动排序设置位置算法
 * @param {}
 */
UNAS.__autoArrangeFun = function(dstpInfo) {
  //根据位置设置图标位置
  for (var k in UNAS.CMGridAlignls) {
    jQuery(document.getElementById(k)).css({
      "top": (UNAS.CMGridAlignls[k] % dstpInfo.IconN) > 0 ? ((UNAS.CMGridAlignls[k] % dstpInfo.IconN) - 1) * dstpInfo.IconY : (dstpInfo.IconN - 1) * dstpInfo.IconY + "px",
      "left": ((Math.ceil(UNAS.CMGridAlignls[k] / dstpInfo.IconN) - 1)) * UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutWidth + "px"
    })
  }
}
/**
 * 桌面右键网格排序设置位置算法
 * @param {}
 */
UNAS.__gridAlignFun = function(dstpInfo) {
  for (var k in UNAS.CMGridAlignls) {
    jQuery(document.getElementById(k)).css({
      "top": (UNAS.CMGridAlignls[k] % dstpInfo.IconN) > 0 ? ((UNAS.CMGridAlignls[k] % dstpInfo.IconN) - 1) * dstpInfo.IconY : (dstpInfo.IconN - 1) * dstpInfo.IconY + "px",
      "left": ((Math.ceil(UNAS.CMGridAlignls[k] / dstpInfo.IconN) - 1)) * UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop].ShortcutWidth + "px"
    })
  }
}
/**
 * create shortcut with contextMenu in apps's translation
 * @param id
 */
UNAS.CSWCMtrans = function(id) {
  var rt = document.getElementById(id + "DesktopShortcut");
  var pageTranslators = window.top.UNAS.CreateNodesTranslators(rt.parentNode);
  var OnChangeLanguage = function(e) {
    for (var i = 0; i < pageTranslators.length; i++) {
      var node = pageTranslators[i].node;
      var p = pageTranslators[i].translateProperty;
      node[p] = window.top.UNAS._(pageTranslators[i].originalText);
    }
  };
  OnChangeLanguage();
  window.top.UNAS.Event.addEvent('ChangeLanguage', OnChangeLanguage);
  window.addEventListener("unload", function(e) {
    window.top.UNAS.Event.removeEvent('ChangeLanguage', OnChangeLanguage);
  }, false);
}

/******************************************************************************************/

/**
 * U-NAS Utilities, Additional, not in the UNAS namespace.
 */
/**
 *Add event handler to an element
 *@param {Element} obj - Element to add event
 *@param {Event} evt - Event
 *@param {Function} fn - Event handler
 */

function addEventSimple(obj, evt, fn, endfn) {
  if (obj.addEventListener) {
    obj.addEventListener(evt, function() {
      if (UNAS.AppCanClick) {
        UNAS.AppCanClick = false;
        setTimeout(function() {
          UNAS.AppCanClick = true;
        }, 250);
        fn();
        endfn && endfn();
      }
    }, false);
  } else if (obj.attachEvent) {
    obj.attachEvent('on' + evt, function() {
      if (UNAS.AppCanClick) {
        UNAS.AppCanClick = false;
        setTimeout(function() {
          UNAS.AppCanClick = true;
        }, 250);
        fn();
        endfn && endfn();
      }
    });
  }
}
/**
 *Remove Event handler
 *@param {Element} obj - Element to remove event
 *@param {Event} evt - Event
 *@param {Function} fn - Event handler
 */
function removeEventSimple(obj, evt, fn) {
  if (obj.removeEventListener)
    obj.removeEventListener(evt, fn, false);
  else if (obj.detachEvent)
    obj.detachEvent('on' + evt, fn);
}

/**
 *Get Cookie by name
 *@param {String} name - Cookie name
 */

function getCookie(name) {
  var arr, reg = new RegExp("(^| )" + name + "=([^;]*)(;|$)");
  if (arr = document.cookie.match(reg))
    return unescape(arr[2]);
  else
    return null;
}
UNAS.DeletVirtualDesktops = function(id) {
  var vd = UNAS.Desktop.VirtualDesktops[UNAS.Desktop.CurrentVirtualDesktop];
  for (var i = 0; i < vd.Shortcuts.length; i++) {
    if (id == vd.Shortcuts[i].id) {
      vd.Shortcuts.splice(i, 1);
      break;
    }
  }
}
//安装后重新排序
UNAS.ResortAfterInstall = function(id, upgrade) {
  var dstpInfo = UNAS.GetDesktopPositionInfo("pageWrapper");
  (id && !upgrade) && UNAS.NewAppAtNewPos(id);
  // UNAS.DeletVirtualDesktops(id);
  if (UNAS.ElementSortType == "autoArrange") {
    // contextMenuFun.__gridGenerateOrderFun(dstpInfo);
    // contextMenuFun.__generateListOrder();
    UNAS.__autoArrangeFun(dstpInfo);
    // UNAS.SaveUserDesktop();
  } else if (UNAS.ElementSortType == "gridAlign") {
    // var dstpInfo = UNAS.GetDesktopPositionInfo("pageWrapper");
    // contextMenuFun.__gridGenerateOrderFun(dstpInfo);
    // UNAS.NewAppAtNewPos(id);
    UNAS.__gridAlignFun(dstpInfo);
  } else if (UNAS.ElementSortType == "ungridAlign") {
    if (UNAS.UngridAlignApps) {
      for (var i = 0; i < UNAS.UngridAlignApps.length; i++) {
        if (UNAS.UngridAlignApps[i].id == id) {
          jQuery(document.getElementById(id)).css({
            "top": UNAS.UngridAlignApps[i].top,
            "left": UNAS.UngridAlignApps[i].left
          })
          break;
        }
      }
    }
  }
  UNAS.SaveUserDesktop();
}
//卸载app后整理变量，重新排序
UNAS.ResortAfterUninstall = function(id, uninstallOrUpgrade) {
  var dstpInfo = UNAS.GetDesktopPositionInfo("pageWrapper");
  var CMGRidNumber = parseInt(UNAS.CMGridAlignls[id]);
  uninstallOrUpgrade == "uninstall" && delete UNAS.CMGridAlignls[id]; //只在卸载时，删除对应值
  //更新或卸载都需要删除此变量内对应的值
  // UNAS.DeletVirtualDesktops(id);
  if (UNAS.ElementSortType == "autoArrange") {
    if (uninstallOrUpgrade == "uninstall") {
      for (var k in UNAS.CMGridAlignls) {
        if (parseInt(UNAS.CMGridAlignls[k]) > CMGRidNumber) {
          UNAS.CMGridAlignls[k] = parseInt(UNAS.CMGridAlignls[k]) - 1;
        }
      }
    }
    UNAS.__autoArrangeFun(dstpInfo);
  } else if (UNAS.ElementSortType == "gridAlign") {
    UNAS.__gridAlignFun(dstpInfo);
  }
  UNAS.SaveUserDesktop();
}
//新安装的app放在第一个空位置,在此分配序号
UNAS.NewAppAtNewPos = function(id) {
  var a = [];
  for (var k in UNAS.CMGridAlignls) {
    var b = [];
    b.push(k, UNAS.CMGridAlignls[k]);
    a.push(b)
  }
  if (UNAS.ElementSortType == "autoArrange") {
    UNAS.CMGridAlignls[id] = a[a.length - 1][1] + 1;
  } else if (UNAS.ElementSortType == "gridAlign") {
    a.sort(function(a, b) {
      return a[1] - b[1]
    });
    for (var i = 1; i < a.length; i++) {
      if (a[i][1] - a[i - 1][1] > 1) {
        UNAS.CMGridAlignls[id] = a[i - 1][1] + 1;
        break;
      }
    }
    if (!UNAS.CMGridAlignls[id]) {
      UNAS.CMGridAlignls[id] = a[a.length - 1][1] + 1;
    }
  }

}
//计算对象的长度
UNAS.CountProperties = function(obj) {
  var count = 0;
  for (var p in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, p)) {
      count++;
    }
  }
  return count;
}
/**
 *UNAS Convert Bytes to readable string (KB, MB ...)
 *@function Conver from bytes to readable String (KB, MB...)
 *@param {num} size - bytes raw speed
 *@return {String} the readable string of size (KB, MB...)
 */
UNAS.getReadableSizeString = function(size) {
  if (size == 0) return "0B"
  var i = -1;
  var byteUnits = ['KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
  do {
    size = size / 1024;
    i++;
  } while (size > 1024);

  return Math.max(size, 0.1).toFixed(1) + byteUnits[i];
};
/**
 *UNAS Convert Speed to readable string (KB/s, MB/s ...)
 *@function Conver Speed from bytes/s to readable String (KB/s, MB/s...)
 *@param {num} size - bytes/s raw speed
 *@return {String} the readable string of speed
 */
UNAS.getReadableSpeedString = function(size) {
  if (size == 0) return "0B/s"
  var i = -1;
  var byteUnits = ['KB/s', 'MB/s', 'GB/s', 'TB/s', 'PB/s', 'EB/s', 'ZB/s', 'YB/s'];
  do {
    size = size / 1024;
    i++;
  } while (size > 1024);

  return Math.max(size, 0.1).toFixed(1) + byteUnits[i];
};

UNAS.is_mobile_unas = function() {
  if (/AppleWebKit.*Mobile/i.test(navigator.userAgent) ||
    /iPhone/i.test(navigator.userAgent) ||
    /Android/i.test(navigator.userAgent) ||
    /BlackBerry/i.test(navigator.userAgent) ||
    /IEMobile/i.test(navigator.userAgent) ||
    (/MIDP|SymbianOS|NOKIA|SAMSUNG|LG|NEC|TCL|Alcatel|BIRD|DBTEL|Dopod|PHILIPS|HAIER|LENOVO|MOT-|Nokia|SonyEricsson|SIE-|Amoi|ZTE/.test(navigator.userAgent))) {
    return true;
  } else {
    return false;
  }
}

function getClientWH_NAS() {
  var clientHeight = 0;
  if (document.body.clientHeight && document.documentElement.clientHeight) {
    clientHeight = Math.min(document.body.clientHeight, document.documentElement.clientHeight);
    clientWidth = Math.min(document.body.clientWidth, document.documentElement.clientWidth);
  } else {
    clientHeight = Math.max(document.body.clientHeight, document.documentElement.clientHeight);
    clientWidth = Math.max(document.body.clientWidth, document.documentElement.clientWidth);
  }

  return {
    w: clientWidth,
    h: clientHeight
  };
}

UNAS.ConfigWizardOobe = function (){
  jQuery.ajax({
      url: "/wmi/index.php",
      type: "POST",
      dataType: 'json',
      data: JSON.encode({
        cmd: 'configwizard_oobe',
        token: UToken,
        params: {
          implemented: 1
        }
      }),
      success: function (data) {
        if (data.err == 0) {
          var info = data.respond || {};
          if (info.implemented) {
            unas_pwdmodified();
          }else {
            ConfigWizard.ConfigWizardAppWindow();
          }
        }
      },
      error: function (XMLHttpRequest, textStatus, errorThrown) {
      }
  });
}
